@@ -26,6 +26,9 @@
 #define CLI_DEFAULT_AC_TRACKLEN	    8
 
 #define CLI_DEFAULT_LSIG_BUFSIZE    32768
+#define CLI_DEFAULT_DBIO_BUFSIZE    CLI_DEFAULT_LSIG_BUFSIZE + 1
+
+#define CLI_DEFAULT_BM_OFFMODE_FSIZE	262144
 
 #define CLI_DEFAULT_MAXSCANSIZE	    104857600
 #define CLI_DEFAULT_MAXFILESIZE	    26214400
