@@ -53,6 +53,7 @@
 typedef enum {
     HTML_BAD_STATE,
     HTML_NORM,
+    HTML_8BIT,
     HTML_COMMENT,
     HTML_CHAR_REF,
     HTML_ENTITY_REF_DECODE,
@@ -470,10 +471,40 @@ void html_tag_arg_free(tag_arguments_t *tags)
 static inline void html_tag_contents_append(struct tag_contents *cont, const unsigned char* begin,const unsigned char *end)
 {
 	size_t i;
+        uint32_t mbchar = 0;
 	if(!begin || !end)
 		return;
 	for(i = cont->pos; i < MAX_TAG_CONTENTS_LENGTH && (begin < end);i++) {
-		cont->contents[i] = *begin++;
+            uint8_t c = *begin++;
+            if (mbchar && (c < 0x80 || mbchar >= 0x10000)) {
+                if (mbchar == 0xE38082 || mbchar == 0xEFBC8E
+                    || mbchar == 0xEFB992 ||
+                    (mbchar == 0xA1 && (c == 0x43 || c == 0x44 || c == 0x4F))) {
+                    cont->contents[i++] = '.';
+                    if (mbchar == 0xA1) {
+                        --i;
+                        mbchar = 0;
+                        continue;
+                    }
+                } else {
+                    uint8_t c0 = mbchar >> 16;
+                    uint8_t c1 = (mbchar >> 8)&0xff;
+                    uint8_t c2 = (mbchar & 0xff);
+                    if (c0 && i+1 < MAX_TAG_CONTENTS_LENGTH)
+                        cont->contents[i++] = c0;
+                    if ((c0 || c1) && i+1 < MAX_TAG_CONTENTS_LENGTH)
+                        cont->contents[i++] = c1;
+                    if (i+1 < MAX_TAG_CONTENTS_LENGTH)
+                        cont->contents[i++] = c2;
+                }
+                mbchar = 0;
+            }
+            if (c >= 0x80) {
+                mbchar = (mbchar << 8) | c;
+                --i;
+            }
+            else
+		cont->contents[i] = c;
 	}
 	cont->pos = i;
 }
@@ -631,6 +662,8 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 	struct parser_state *js_state = NULL;
 	const unsigned char *js_begin = NULL, *js_end = NULL;
 	struct tag_contents contents;
+        uint32_t mbchar = 0;
+        uint32_t mbchar2 = 0;
 
 	tag_args.scanContents=0;/* do we need to store the contents of <a></a>?*/
 	contents.pos = 0;
@@ -749,6 +782,42 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 					next_state = HTML_BAD_STATE;
 				}
 				break;
+                        case HTML_8BIT:
+                                if (*ptr < 0x80 || mbchar >= 0x10000) {
+                                    if (mbchar == 0xE38082 || mbchar == 0xEFBC8E
+                                        || mbchar == 0xEFB992 ||
+                                        (mbchar == 0xA1 && (*ptr == 0x43 || *ptr == 0x44 || *ptr == 0x4F))) {
+                                        /* bb #4097 */
+                                        html_output_c(file_buff_o2, '.');
+                                        html_output_c(file_buff_text, '.');
+                                        if (mbchar == 0xA1) {
+                                            ptr++;
+                                            mbchar = 0;
+                                            continue;
+                                        }
+                                    } else {
+                                        uint8_t c0 = mbchar >> 16;
+                                        uint8_t c1 = (mbchar >> 8)&0xff;
+                                        uint8_t c2 = (mbchar & 0xff);
+                                        if (c0) {
+                                            html_output_c(file_buff_o2, c0);
+                                            html_output_c(file_buff_text, c0);
+                                        }
+                                        if (c0 || c1) {
+                                            html_output_c(file_buff_o2, c1);
+                                            html_output_c(file_buff_text, c1);
+                                        }
+                                        html_output_c(file_buff_o2, c2);
+                                        html_output_c(file_buff_text, c1);
+                                    }
+                                    mbchar = 0;
+                                    state = next_state;
+                                    next_state = HTML_NORM;
+                                } else {
+                                    mbchar = (mbchar << 8) | *ptr;
+                                    ptr++;
+                                }
+                                break;
 			case HTML_NORM:
 				if (*ptr == '<') {
 					ptrend=ptr; /* for use by scanContents */
@@ -781,6 +850,11 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 					state = HTML_CHAR_REF;
 					next_state = HTML_NORM;
 					ptr++;
+                                } else if (*ptr >= 0x80) {
+                                        state = HTML_8BIT;
+                                        next_state = HTML_NORM;
+                                        mbchar = *ptr;
+                                        ptr++;
 				} else {
 					unsigned char c = tolower(*ptr);
 					/* normalize ' to " for scripts */
@@ -1007,11 +1081,45 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 						ptr++;
 					}
 				} else {
+                                    if (mbchar2 && (*ptr < 0x80 || mbchar2 >= 0x10000)) {
+                                        if (mbchar2 == 0xE38082 || mbchar2 == 0xEFBC8E
+                                            || mbchar2 == 0xEFB992 ||
+                                            (mbchar2 == 0xA1 && (*ptr == 0x43 || *ptr == 0x44 || *ptr == 0x4F))) {
+                                            html_output_c(file_buff_o2, '.');
+                                            if (tag_val_length < HTML_STR_LENGTH)
+						tag_val[tag_val_length++] = '.';
+                                            if (mbchar2 == 0xA1) {
+                                                ptr++;
+                                                mbchar2 = 0;
+                                                continue;
+                                            }
+                                        } else {
+                                            uint8_t c0 = mbchar2 >> 16;
+                                            uint8_t c1 = (mbchar2 >> 8)&0xff;
+                                            uint8_t c2 = (mbchar2 & 0xff);
+                                            if (c0)
+                                                html_output_c(file_buff_o2, c0);
+                                            if (c0 || c1)
+                                                html_output_c(file_buff_o2, c1);
+                                            html_output_c(file_buff_o2, c2);
+                                            if (c0 && tag_val_length < HTML_STR_LENGTH)
+						tag_val[tag_val_length++] = c0;
+                                            if ((c0 || c1) && tag_val_length < HTML_STR_LENGTH)
+						tag_val[tag_val_length++] = c1;
+                                            if (tag_val_length < HTML_STR_LENGTH)
+						tag_val[tag_val_length++] = c2;
+					}
+                                        mbchar2 = 0;
+                                    }
+                                    if (*ptr >= 0x80)
+                                        mbchar2 = (mbchar2 << 8) | *ptr;
+                                    else {
 					html_output_c(file_buff_o2, tolower(*ptr));
 					if (tag_val_length < HTML_STR_LENGTH) {
 						tag_val[tag_val_length++] = *ptr;
 					}
-					ptr++;
+                                    }
+				    ptr++;
 				}
 
 				if (*ptr == '\\') {
