@@ -538,6 +538,8 @@ static int cli_loaddb(FILE *fs, struct cl_engine *engine, unsigned int *signo, u
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
+	if(buffer[0] == '#')
+	    continue;
 	cli_chomp(buffer);
 	if(engine->ignored)
 	    strcpy(buffer_cpy, buffer);
@@ -884,6 +886,8 @@ static int cli_loadndb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
+	if(buffer[0] == '#')
+	    continue;
 
 	if(!phish)
 	    if(!strncmp(buffer, "HTML.Phishing", 13) || !strncmp(buffer, "Email.Phishing", 14))
@@ -1192,7 +1196,7 @@ static int lsigattribs(char *attribs, struct cli_lsig_tdb *tdb)
   } while(0);
 
 #define LDB_TOKENS 67
-static int load_oneldb(char *buffer, int chkpua, int chkign, struct cl_engine *engine, unsigned int options, const char *dbname, unsigned int line, unsigned int *sigs, unsigned bc_idx, const char *buffer_cpy)
+static int load_oneldb(char *buffer, int chkpua, struct cl_engine *engine, unsigned int options, const char *dbname, unsigned int line, unsigned int *sigs, unsigned bc_idx, const char *buffer_cpy, int *skip)
 {
     const char *sig, *virname, *offset, *logic;
     struct cli_ac_lsig **newtable, *lsig;
@@ -1214,8 +1218,11 @@ static int load_oneldb(char *buffer, int chkpua, int chkign, struct cl_engine *e
     if (chkpua && cli_chkpua(virname, engine->pua_cats, options))
 	    return CL_SUCCESS;
 
-    if (chkign && cli_chkign(engine->ignored, virname, buffer_cpy))
+    if (engine->ignored && cli_chkign(engine->ignored, virname, buffer_cpy ? buffer_cpy : virname)) {
+	if(skip)
+	    *skip = 1;
 	return CL_SUCCESS;
+    }
 
     if(engine->cb_sigload && engine->cb_sigload("ldb", virname, engine->cb_sigload_ctx)) {
 	cli_dbgmsg("cli_loadldb: skipping %s due to callback\n", virname);
@@ -1375,6 +1382,8 @@ static int cli_loadldb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    return CL_EMEM;
     while(cli_dbgets(buffer, sizeof(buffer), fs, dbio)) {
 	line++;
+	if(buffer[0] == '#')
+	    continue;
 	sigs++;
 	cli_chomp(buffer);
 
@@ -1382,8 +1391,7 @@ static int cli_loadldb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    strcpy(buffer_cpy, buffer);
 	ret = load_oneldb(buffer,
 			  engine->pua_cats && (options & CL_DB_PUA_MODE) && (options & (CL_DB_PUA_INCLUDE | CL_DB_PUA_EXCLUDE)),
-			  !!engine->ignored,
-			  engine, options, dbname, line, &sigs, 0, buffer_cpy);
+			  engine, options, dbname, line, &sigs, 0, buffer_cpy, NULL);
 	if (ret)
 	    break;
     }
@@ -1409,7 +1417,7 @@ static int cli_loadldb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigned int options, struct cli_dbio *dbio, const char *dbname)
 {
     char buf[4096];
-    int rc;
+    int rc, skip = 0;
     struct cli_all_bc *bcs = &engine->bcs;
     struct cli_bc *bc;
     unsigned sigs = 0;
@@ -1470,7 +1478,6 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
     bc->id = bcs->count;/* must set after _load, since load zeroes */
     if (engine->bytecode_mode == CL_BYTECODE_MODE_TEST)
 	cli_infomsg(NULL, "bytecode %u -> %s\n", bc->id, dbname);
-    sigs++;
     if (bc->kind == BC_LOGICAL || bc->lsig) {
         unsigned oldsigs = sigs;
 	if (!bc->lsig) {
@@ -1478,12 +1485,17 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    return CL_EMALFDB;
 	}
 	cli_dbgmsg("Bytecode %s(%u) has logical signature: %s\n", dbname, bc->id, bc->lsig);
-	rc = load_oneldb(bc->lsig, 0, 0, engine, options, dbname, 0, &sigs, bcs->count, NULL);
+	rc = load_oneldb(bc->lsig, 0, engine, options, dbname, 0, &sigs, bcs->count, NULL, &skip);
 	if (rc != CL_SUCCESS) {
 	    cli_errmsg("Problem parsing logical signature %s for bytecode %s: %s\n",
 		       bc->lsig, dbname, cl_strerror(rc));
 	    return rc;
 	}
+	if (skip) {
+	    cli_bytecode_destroy(bc);
+	    bcs->count--;
+	    return CL_SUCCESS;
+	}
         if (sigs != oldsigs) {
           /* compiler ensures Engine field in lsig matches the one in bytecode,
            * so this should never happen. */
@@ -1491,6 +1503,7 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
           return CL_EMALFDB;
         }
     }
+    sigs++;
     if (bc->kind != BC_LOGICAL) {
 	if (bc->lsig) {
 	    /* runlsig will only flip a status bit, not report a match,
@@ -1557,6 +1570,8 @@ static int cli_loadftm(FILE *fs, struct cl_engine *engine, unsigned int options,
 	} else {
 	    if(!cli_dbgets(buffer, FILEBUFF, fs, dbio))
 		break;
+	    if(buffer[0] == '#')
+		continue;
 	    cli_chomp(buffer);
 	}
 	line++;
@@ -1794,6 +1809,8 @@ static int cli_loadign(FILE *fs, struct cl_engine *engine, unsigned int options,
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
+	if(buffer[0] == '#')
+	    continue;
 	cli_chomp(buffer);
 
 	tokens_count = cli_strtokenize(buffer, ':', IGN_MAX_TOKENS + 1, tokens);
@@ -1883,6 +1900,8 @@ static int cli_loadhash(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
+	if(buffer[0] == '#')
+	    continue;
 	cli_chomp(buffer);
 	if(engine->ignored)
 	    strcpy(buffer_cpy, buffer);
@@ -2677,7 +2696,7 @@ const char *cl_retdbdir(void)
 int cl_statinidir(const char *dirname, struct cl_stat *dbstat)
 {
 	DIR *dd;
-	const struct dirent *dent;
+	struct dirent *dent;
 #if defined(HAVE_READDIR_R_3) || defined(HAVE_READDIR_R_2)
 	union {
 	    struct dirent d;
