@@ -766,6 +766,37 @@ static int cli_scanmscab(int desc, cli_ctx *ctx, off_t sfx_offset)
     return ret;
 }
 
+static int vba_scandata(const unsigned char *data, unsigned int len, cli_ctx *ctx)
+{
+	struct cli_matcher *groot = ctx->engine->root[0];
+	struct cli_matcher *troot = ctx->engine->root[2];
+	struct cli_ac_data gmdata, tmdata;
+	struct cli_ac_data *mdata[2];
+	int ret;
+
+    if((ret = cli_ac_initdata(&tmdata, troot->ac_partsigs, troot->ac_lsigs, troot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)))
+	return ret;
+
+    if((ret = cli_ac_initdata(&gmdata, groot->ac_partsigs, groot->ac_lsigs, groot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN))) {
+	cli_ac_freedata(&tmdata);
+	return ret;
+    }
+    mdata[0] = &tmdata;
+    mdata[1] = &gmdata;
+
+    ret = cli_scanbuff(data, len, 0, ctx, CL_TYPE_MSOLE2, mdata);
+
+    if(ret != CL_VIRUS) {
+	ret = cli_lsig_eval(ctx, troot, &tmdata, NULL, NULL);
+	if(ret != CL_VIRUS)
+	    ret = cli_lsig_eval(ctx, groot, &gmdata, NULL, NULL);
+    }
+    cli_ac_freedata(&tmdata);
+    cli_ac_freedata(&gmdata);
+
+    return ret;
+}
+
 static int cli_vba_scandir(const char *dirname, cli_ctx *ctx, struct uniq *U)
 {
 	int ret = CL_CLEAN, i, j, fd, data_len, hasmacros = 0;
@@ -806,7 +837,7 @@ static int cli_vba_scandir(const char *dirname, cli_ctx *ctx, struct uniq *U)
 		    /* cli_dbgmsg("Project content:\n%s", data); */
 		    if(ctx->scanned)
 			*ctx->scanned += data_len / CL_COUNT_PRECISION;
-		    if(cli_scanbuff(data, data_len, 0, ctx, CL_TYPE_MSOLE2, NULL) == CL_VIRUS) {
+		    if(vba_scandata(data, data_len, ctx) == CL_VIRUS) {
 			free(data);
 			ret = CL_VIRUS;
 			break;
@@ -831,7 +862,6 @@ static int cli_vba_scandir(const char *dirname, cli_ctx *ctx, struct uniq *U)
 	    fd = open(vbaname, O_RDONLY|O_BINARY);
 	    if (fd == -1) continue;
 	    if ((fullname = cli_ppt_vba_read(fd, ctx))) {
-		hasmacros++;
 		if(cli_scandir(fullname, ctx) == CL_VIRUS) {
 		    ret = CL_VIRUS;
 		}
@@ -858,14 +888,13 @@ static int cli_vba_scandir(const char *dirname, cli_ctx *ctx, struct uniq *U)
 	    for (i = 0; i < vba_project->count; i++) {
 		cli_dbgmsg("VBADir: Decompress WM project macro:%d key:%d length:%d\n", i, vba_project->key[i], vba_project->length[i]);
 		data = (unsigned char *)cli_wm_decrypt_macro(fd, vba_project->offset[i], vba_project->length[i], vba_project->key[i]);
-		hasmacros++;
 		if(!data) {
 			cli_dbgmsg("VBADir: WARNING: WM project '%s' macro %d decrypted to NULL\n", vba_project->name[i], i);
 		} else {
 			cli_dbgmsg("Project content:\n%s", data);
 			if(ctx->scanned)
 			    *ctx->scanned += vba_project->length[i] / CL_COUNT_PRECISION;
-			if(cli_scanbuff(data, vba_project->length[i], 0, ctx, CL_TYPE_MSOLE2, NULL) == CL_VIRUS) {
+			if(vba_scandata(data, vba_project->length[i], ctx) == CL_VIRUS) {
 				free(data);
 				ret = CL_VIRUS;
 				break;
@@ -1936,7 +1965,7 @@ static void emax_reached(cli_ctx *ctx) {
 
 static int magic_scandesc(int desc, cli_ctx *ctx, cli_file_t type)
 {
-	int ret = CL_CLEAN;
+	int ret = CL_CLEAN, res;
 	cli_file_t dettype = 0;
 	struct stat sb;
 	uint8_t typercg = 1;
@@ -2009,10 +2038,11 @@ static int magic_scandesc(int desc, cli_ctx *ctx, cli_file_t type)
 	}
     }
 
-    if(cache_check(hash, ctx) == CL_CLEAN) {
+    res = cache_check(hash, ctx);
+    if(res != CL_VIRUS) {
 	funmap(*ctx->fmap);
 	ctx->fmap--;
-	ret_from_magicscan(CL_CLEAN);
+	ret_from_magicscan(res);
     }
     hashed_size = (*ctx->fmap)->len;
     old_hook_lsig_matches = ctx->hook_lsig_matches;
@@ -2329,8 +2359,12 @@ static int magic_scandesc(int desc, cli_ctx *ctx, cli_file_t type)
 
     /* CL_TYPE_HTML: raw HTML files are not scanned, unless safety measure activated via DCONF */
     if(type != CL_TYPE_IGNORED && (type != CL_TYPE_HTML || !(DCONF_DOC & DOC_CONF_HTML_SKIPRAW)) && !ctx->engine->sdb) {
-	if(cli_scanraw(ctx, type, typercg, &dettype, hash) == CL_VIRUS) {
-	    ret =  cli_checkfp(hash, hashed_size, ctx);
+	res = cli_scanraw(ctx, type, typercg, &dettype, hash);
+	if(res != CL_CLEAN) {
+	    if(res == CL_VIRUS)
+		ret =  cli_checkfp(hash, hashed_size, ctx);
+	    else
+		ret = res;
 	    funmap(*ctx->fmap);
 	    ctx->fmap--;
 	    cli_bitset_free(ctx->hook_lsig_matches);
