@@ -45,6 +45,7 @@
 
 #include "filetypes.h"
 #include "textdet.h"
+#include "others.h"
 
 #define F 0   /* character never appears in text */
 #define T 1   /* character appears in plain ASCII text */
@@ -141,7 +142,7 @@ static int td_isutf8(const unsigned char *buf, unsigned int len)
 
 static int td_isutf16(const unsigned char *buf, unsigned int len)
 {
-	unsigned int be, i, c;
+	unsigned int be = 1, nobom = 0, i, c, bad = 0;
 
 
     if(len < 2)
@@ -152,7 +153,7 @@ static int td_isutf16(const unsigned char *buf, unsigned int len)
     else if(buf[0] == 0xfe && buf[1] == 0xff)
 	be = 1;
     else
-	return 0;
+	nobom = 1;
 
     for(i = 2; i + 1 < len; i += 2) {
 	if(be)
@@ -163,10 +164,17 @@ static int td_isutf16(const unsigned char *buf, unsigned int len)
 	if(c == 0xfffe)
 	    return 0;
 
-	if(c < 128 && text_chars[c] != T)
-	    return 0;
+	if(c < 128 && text_chars[c] != T) {
+	    if(nobom)
+		return 0;
+	    else
+		bad++;
+	}
     }
 
+    if(!nobom && bad >= len / 2)
+	return 0;
+
     return 1 + be;
 }
 
@@ -174,16 +182,17 @@ cli_file_t cli_texttype(const unsigned char *buf, unsigned int len)
 {
 	int ret;
 
-    if(td_isutf8(buf, len)) {
+    if(td_isascii(buf, len)) {
+	cli_dbgmsg("Recognized ASCII text\n");
+	return CL_TYPE_TEXT_ASCII;
+    } else if(td_isutf8(buf, len)) {
+	cli_dbgmsg("Recognized UTF-8 character data\n");
 	return CL_TYPE_TEXT_UTF8;
     } else if((ret = td_isutf16(buf, len))) {
-	if(ret == 1)
-	    return CL_TYPE_TEXT_UTF16LE;
-	else
-	    return CL_TYPE_TEXT_UTF16BE;
-    } else if(td_isascii(buf, len)) {
-	return CL_TYPE_TEXT_ASCII;
+	cli_dbgmsg("Recognized %s character data\n", (ret == 1) ? "UTF-16LE" : "UTF-16BE");
+	return (ret == 1) ? CL_TYPE_TEXT_UTF16LE : CL_TYPE_TEXT_UTF16BE;
     } else {
+	cli_dbgmsg("Recognized binary data\n");
 	return CL_TYPE_BINARY_DATA;
     }
 }
