@@ -71,7 +71,7 @@
 #include "libclamav/readdb.h"
 #include "libclamav/others.h"
 
-#define MAX_DEL_LOOKAHEAD   200
+#define MAX_DEL_LOOKAHEAD   5000
 
 static const struct dblist_s {
     const char *ext;
@@ -500,7 +500,7 @@ static int script2cdiff(const char *script, const char *builder, const struct op
 	SHA256_CTX ctx;
 	struct stat sb;
 	FILE *scripth, *cdiffh;
-	gzFile *gzh;
+	gzFile gzh;
 	unsigned int ver, osize;
 	int bytes;
 
@@ -629,7 +629,7 @@ static int build(const struct optstruct *opts)
 	const char *newcvd, *localdbdir = NULL;
         struct cl_engine *engine;
 	FILE *cvd, *fh;
-	gzFile *tar;
+	gzFile tar;
 	time_t timet;
 	struct tm *brokent;
 	struct cl_cvd *oldcvd;
